% VAL = BILINEAR_INTERPOLATION(I,X,Y) 
% takes an imput image I and a two non-integer coordiantes X,Y and computes
% the bilinear interpolation at the point I(X,Y).
% For teaching purposes only. Extremely ineffiencient
%
% INPUT:    I   input gray value image
%           X,Y non-integer coordiantes, where the interpolation is to be
%               performed
%
% OUTPUT:   VAL interpolated value of I at (X,Y)
%

% created by darko zikic (zikic@in.tum.de) 
% 2006-11-08
% for Lecture Basic Mathematical Tools for Imaging and Visualization 
% at Chair for Computer Aided Medical Procedures & Augmented Reality
% Department for Computer Science, Technische Universitt Mnchen
% Winter Term 2006

function val = bilinear_interpolation(I,x,y)
 
% get sizes of image
[n1,n2] = size(I);

% compute floor and ceil
xf = floor(x);
xc = ceil(x);
yf = floor(y);
yc = ceil(y);

if (xf==xc && yf==yc) % if x and y are integer values

    % just return the value at the grid point
    val = I(xf,yf);
    
elseif (xf==xc) % if x is integer

    % linear interpolation for y    
    val = I(xf,yf)*(yc-y) + I(xf,yc)*(y-yf) ;

elseif (yf==yc) % if x is integer

    % linear interpolation for x 
    val = I(xf,yf)*(xc-x) + I(xc,yf)*(x-xf);

else % 'normal' case

    val =   I(xf,yf)*(xc-x)*(yc-y) ...
            + I(xc,yf)*(x-xf)*(yc-y) ...
            + I(xf,yc)*(xc-x)*(y-yf) ...
            + I(xc,yc)*(x-xf)*(y-yf) ;
end

return;