% I_H = TRANSFORM_IMAGE(I,H,IP) 
% takes an imput image I and a transformation matrix H and returns the
% transformed image I_H computed by backward warping
%
% INPUT:    I   input gray value image
%           H   transformation matrix 3x3 (homogeneous coordinates)
%           ip  interpolation method: 
%               'nearest' for nearest neighbour
%               'bilinear' for bilinear interpolation (default)
%
% OUTPUT:   I_H   transformed image
%

function I_H = transform_image_backward(I,H,interpolation_method)

% use bilinear interpolation as default
if nargin<3
    interpolation_method = 'bilinear';
end

% get sizes of image
[height , width , channels] = size(I);

% create output image of size 
I_H = zeros(size(I));

% precompute the inverse of the transformation matrix H


% go through all pixels of the result image I_H.
% compute the corresponding position in input image I,
% and write the corresponding intensity to I_H
for channel = 1:channels % do all colours
    
    sub_image = I(:, :, channel);
    
    for v_H = 1:height
        for u_H = 1:width

            % compute position of the pixel p=[u,v,1] (p in input image I) 
            % that maps to pixel p'=[u_H, v_H, 1] in output image I_H
            % It holds that Hp=p' (p from input image I)
            % (remember use homogeneous coordinates)
            
            % write the intensity of I(u,v) to I_H(u_H,v_H)
            % (use only values inside the image domain)

            % for switching between the two different interpolation methods,
            % you can use the following switch
            switch interpolation_method

                case 'nearest' % nearest neighbour interpolation               
                    fprintf('using nearest neaighbor interpolation \n');

                case 'bilinear' % bilinear interpolation            
                    fprintf('using bilinear interpolation \n');

            end        
        end    
    end
end

% make sure that output has same data type as input image
I_H = cast(I_H,class(I));

return;